{
Copyright 2006 Janis Elsts AKA White Shadow
http://w-shadow.com/

Free for non-commercial use.

Requires some additional libraries :
  DelphiNEAT (http://www.cambrianlabs.com/mattias/DelphiNEAT/)
  PNGImage (http://pngdelphi.sourceforge.net/)
  NativeXML (shareware, do a Google search ;P)

You'll also need to slightly modify uNeatToXML (from DelphiNEAT) to
work with NativeXML which is the newer version of the library
it originally used.
 
Partially based on a DelphiNeat sample app GeneticArt.  
}

unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, uNeatClasses, uNeatToXML, NativeXML, uTransferFunctionClasses,
  StdCtrls, Buttons, XPMan, ComCtrls, JPEG, PNGImage;

const
 cDefaultImageWidth=800;
 cDefaultImageHeight=600;

type
  TfmMain = class(TForm)
    Population: TNEATPopulation;
    mmLog: TMemo;
    BitBtn1: TBitBtn;
    mmVotes: TMemo;
    XPManifest1: TXPManifest;
    bbNextGen: TBitBtn;
    BitBtn2: TBitBtn;
    sbStatus: TStatusBar;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    cbImageSize: TComboBox;
    cbSubsamples: TComboBox;
    Label1: TLabel;
    btStop: TButton;
    procedure FormCreate(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure bbNextGenClick(Sender: TObject);
    function PopulationCalculateFitness(NEATPopulation: TNEATPopulation;
      Genotype: TGenotype): Double;
    procedure BitBtn2Click(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
    procedure cbImageSizeChange(Sender: TObject);
    procedure BitBtn4Click(Sender: TObject);
    procedure cbSubsamplesChange(Sender: TObject);
    procedure btStopClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure Log(const aText:string);
    procedure Restart;
    function LoadPopulationFromXML(const FileName:string):boolean;
    function SavePopulationToXML(const FileName:string):boolean;
    function LoadVotes(const xVotes:TStrings):boolean;
    Procedure GenerateIMGandSQL;
  end;

var
  fmMain: TfmMain;
  thisGeneration, Subsamples:integer;
  TotalVotes:cardinal;
  Votes:array of cardinal;
  cImageWidth, cImageHeight:integer;
  ArrR:array of double;
  ArrG:array of double;
  ArrB:array of double;

  Working:boolean;
implementation

{$R *.dfm}

function mclean(x : double) : double;
begin
  //This is one way to normalize the values returned by
  //the neural network. Taken from a DelphiNeat sample application.
  //Not currently used.
  {if x>1 then result:=1
   else if x<-1 then result:=-1
    else result:=x;

  result:=abs(result); //}
  while abs(x)>100 do
    x := x/100;

  while x>1 do
    x:=2-x;

  while x<-1 do
    x:=2+x;

  result := abs(x);//}
end;

procedure TfmMain.Restart;
var
  i,j  : integer;
  Connect : TConnect;
  NewNode: TNode;//}
begin
  with Population do
  begin
    //FitnessMonitor.ResetForNewRun;
    //FitnessMonitor.SetRunState(true);

    // Clear out all old innovations
    DeleteAllConnectInnovations;

    CreatePopulation;

    for i := 0 to Population.PopulationSize-1 do
      with Population.Genotypes[i] do
      begin
        for j := 0 to NodeList.Count-1 do
          NodeList[j].MutateFunction;

        Connect := GetRandomActiveConnect;
        if Assigned(Connect) then
        begin
          NewNode := Connect.MutateSplit;

          if Assigned(NewNode) then
            AddConnection(
              NewNode.NodeID,
              NewNode.NodeID,
              (random*2-1) * 0.1);
        end;

        AddConnection(
          OutputNodes[0].NodeID,
          OutputNodes[0].NodeID,
          (random*2-1) * 0.1); //}

        PreparePhenotype;
      end;//}
  end;
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
  cImageWidth:=200;
  cImageHeight:=150;
  Subsamples:=1;
  Randomize;
  RegisterNewTransferFuntion('Misc', 'Gaussian', TTransferFunctionGaussian.Create);
  //RegisterNewTransferFuntion('Misc', 'IF', TTransferFunctionIF.Create);
  RegisterNewTransferFuntion('Misc', 'DelayGaussian', TTransferFunction1DelayGaussian.Create);
  RegisterNewTransferFuntion('Misc', 'Sigmoind', TTransferFunctionSigmoid.Create);

  //these functions are likely to cause floating point overflow
  //RegisterNewTransferFuntion('Misc', 'Multiply', TTransferFunctionMUL.Create);
  //RegisterNewTransferFuntion('Misc', 'Max', TTransferFunctionMax.Create);
  //RegisterNewTransferFuntion('Misc', 'Min', TTransferFunctionMin.Create);
end;

function TfmMain.LoadPopulationFromXML(const FileName:string):boolean;
var
 FDoc:TNativeXML;
 i,j:integer;
 GenotypeNode : TXMLNode;
begin
 result:=false;
 if FileExists(FileName) then begin
    FDoc:=TNativeXML.Create;
    FDoc.LoadFromFile(FileName);
    if not SameText(Fdoc.Root.Name,'Genotypes') then exit;
    thisGeneration:=FDoc.Root.ReadAttributeInteger('generation',1);
    Log('Loading '+inttostr(FDoc.Root.NodeCount)+' genotypes...');
    for i := 0 to FDoc.Root.NodeCount-1 do
      if SameText(FDoc.Root.Nodes[i].Name, 'Genotype') then
      begin
        GenotypeNode:=Fdoc.Root.Nodes[i];
        j:=GenotypeNode.ReadAttributeInteger('index',i);
        LoadGenotypeFromXMLNode(Population.Genotypes.Items[j],GenotypeNode);
        Population.Genotypes.Items[j].Tag:=j;
      end;
   result:=true;   
 end;
end;

procedure TfmMain.Log(const aText:string);
begin
 mmLog.Lines.add(aText);
 mmLog.SelStart:=Perform(EM_LINEINDEX, mmLog.Lines.Count, 0); 
end;

procedure TfmMain.BitBtn1Click(Sender: TObject);
begin
 Log('Creating a new population...');
 ThisGeneration:=1;
 Restart;
 SavePopulationToXML('genotypes.xml');
 GenerateImgAndSQL;
end;

function tfmMain.LoadVotes(const xVotes:TStrings):boolean;
var
 i,j:integer;
begin
 result:=false;
 if xVotes.Count<Population.PopulationSize then exit;
 xVotes.NameValueSeparator:='=';
 TotalVotes:=0;
 SetLength(Votes,Population.PopulationSize);
 for i:=0 to Population.PopulationSize-1 do begin
  j:=strtoint(xVotes.names[i]);
  Votes[j]:=strtoint(xVotes.ValueFromIndex[i]);
  TotalVotes:=TotalVotes+Votes[j];
 end;
 inc(TotalVotes);
 result:=true; 
end;

procedure TfmMain.bbNextGenClick(Sender: TObject);
begin
 if not LoadVotes(mmVotes.Lines) then begin
  log('The number of votes doesn''t match the number of genotypes!');
  exit;
 end;
 Log('Votes loaded.');

 Population.CalculateFitnesses;
 Population.CreateNewGeneration;
 Log('Genotype generation complete.');

 //save the previous generation
 CopyFile('genotypes.xml',
  Pansichar('oldxml\genotypes-g'+inttostr(ThisGeneration)+'.xml'),
  false);
 inc(ThisGeneration);

 //theoretically it should save to genotypes.xml, but better
 //safe than sorry. When you're satisfied with the new generation,
 //replace genotypes.xml with new_genotypes.xml
 SavePopulationToXML('new_genotypes.xml');

 Log('Generating images and SQL...');
 GenerateImgAndSQL;
end;

function TfmMain.PopulationCalculateFitness(
  NEATPopulation: TNEATPopulation; Genotype: TGenotype): Double;
begin
 //fitness = myvotes/totalvotes
 Result:=Votes[population.CurrentGenotypeID]/TotalVotes;
 Genotype.Fitness:=Result;
end;

function TfmMain.SavePopulationToXML(const FileName:string):boolean;
var
 FDoc:TNativeXML;
 i:integer;
 GenotypeNode : TXMLNode;
begin
 if FileExists(FileName) then DeleteFile(FileName);
 FDoc:=TNativeXML.CreateName('Genotypes');
 FDoc.Root.WriteAttributeInteger('generation',ThisGeneration);
 for i := 0 to Population.PopulationSize-1 do
   begin
     GenotypeNode:=Fdoc.Root.NodeNew('Genotype');
     GenotypeNode.WriteAttributeInteger('index',i);
     SaveGenotypeToXMLNode(Population.Genotypes.Items[i],GenotypeNode);
   end;
 FDoc.SaveToFile(FileName);
 result:=true;  
end;

procedure GenerateImage(const aGenotype:TGenotype; const FileName:string;
  RenderWidth, RenderHeight:integer);
var
 bmp:TBitmap;
 png:TPNGObject;
  x, y, i : integer;
  varx, vary, wdiv, hdiv : single;
  r,g,b ,sr, sg, sb, vx, vy: double;
  q : single;
  d, MaxRad : single;
 MaxColor:double; 
begin
 //this is the interesting part. and it's mostly undocumented. yay! :P
 aGenotype.Flush; //...
 bmp:=TBitmap.Create;
 bmp.Width:=RenderWidth;
 bmp.Height:=RenderHeight;
 MaxColor:=0;

 wdiv := 2/RenderWidth;
 hdiv := 2/RenderHeight;
 MaxRad := 1/Sqrt(2);

 Q := RenderWidth / RenderHeight;

 AGenotype.InputNodes[0].BiasNode := true;

  for y := 0 to RenderHeight-1 do
  for x := 0 to RenderWidth-1 do begin
    begin
      sr:=0; sg:=0; sb:=0;
      for i:=1 to Subsamples do begin
       //subsampling - calculate the color at some random points
       //within a given range and use the average.
       vx:=x+random;
       vy:=y+random;
       varx := (vx*wdiv-1)*q;
       vary := vy*hdiv-1;
       d := sqrt(sqr(varx)+sqr(vary))/MaxRad;

       AGenotype.Flush;

       AGenotype.Inputs[0]:=1;
       AGenotype.Inputs[1]:=varx;
       AGenotype.Inputs[2]:=vary;
       AGenotype.Inputs[3]:=d;
       AGenotype.Inputs[4] := 1; //r
       AGenotype.Inputs[5] := 0; //g
       AGenotype.Inputs[6] := 0; //b
       AGenotype.Iterate(-1);
       r := AGenotype.Outputs[0];

       AGenotype.Inputs[0]:=1;
       AGenotype.Inputs[1]:=varx;
       AGenotype.Inputs[2]:=vary;
       AGenotype.Inputs[3]:=d;
       AGenotype.Inputs[4] := 0;
       AGenotype.Inputs[5] := 1;
       AGenotype.Inputs[6] := 0;
       AGenotype.Iterate(-1);
       g := AGenotype.Outputs[0];

       AGenotype.Inputs[0]:=1;
       AGenotype.Inputs[1]:=varx;
       AGenotype.Inputs[2]:=vary;
       AGenotype.Inputs[3]:=d;
       AGenotype.Inputs[4] := 0;
       AGenotype.Inputs[5] := 0;
       AGenotype.Inputs[6] := 1;
       AGenotype.Iterate(-1);
       b := AGenotype.Outputs[0]; //}

       sr:=sr+mclean(r) * 255;
       sg:=sg+mclean(g) * 255;
       sb:=sb+mclean(b) * 255;
      end;

      r:=sr / Subsamples;
      g:=sg / Subsamples;
      b:=sb / Subsamples;

      ArrR[cImageWidth*y+x]:=r;
      ArrG[cImageWidth*y+x]:=g;
      ArrB[cImageWidth*y+x]:=b;

      if abs(r)>MaxColor then MaxColor:=abs(r);
      if abs(g)>MaxColor then MaxColor:=abs(g);
      if abs(b)>MaxColor then MaxColor:=abs(b);

      {bmp.Canvas.Pixels[x,y]:=
              RGB(
                trunc(r),
                trunc(g),
                trunc(b));//}
    end;
    fmMain.sbStatus.Panels[1].Text:=inttostr(y)+'/'+inttostr(RenderHeight);
    application.ProcessMessages;
 end;
 for x := 0 to cImageWidth-1 do begin
  for y := 0 to cImageHeight-1 do begin
   bmp.Canvas.Pixels[x,y]:=
              RGB(
                trunc(abs(ArrR[cImageWidth*y+x])/MaxColor * 255),
                trunc(abs(ArrG[cImageWidth*y+x])/MaxColor * 255),
                trunc(abs(ArrB[cImageWidth*y+x])/MaxColor * 255))
  end;
 end;//}
 fmMain.sbStatus.Panels[1].Text:='Saving image...';
 PNG := TPNGObject.Create;
 PNG.CompressionLevel:=9;
 //PNG.Filters:=[pfNone, pfSub, pfUp, pfAverage, pfPaeth];
 {In case something goes wrong, free both Bitmap and PNG}
 try
   PNG.Assign(bmp);    //Convert data into png
   PNG.SaveToFile(FileName);
 finally
   bmp.Free;
   PNG.Free;
 end;
 fmMain.sbStatus.Panels[1].Text:='OK';
end;

Procedure tfmmain.GenerateIMGandSQL;
var
 i:integer;
 imgName:string;
 Inserts:TStringList;
begin
 Working:=true;
 BtStop.Visible:=true;
 SetLength(ArrR,cImageWidth*cImageHeight);
 SetLength(ArrG,cImageWidth*cImageHeight);
 SetLength(ArrB,cImageWidth*cImageHeight);
 //create images and SQL insertion code
 Inserts:=TStringList.Create;
 for i:=0 to Population.PopulationSize-1 do begin
  imgName:='li-g'+Format('%.4d',[ThisGeneration])+
    '-s'+Format('%.3d',[i+1])+'.png';
  GenerateImage(Population.Genotypes.Items[i],'images\'+imgName,
   cImageWidth,cImageHeight);
  Inserts.Add(
   'INSERT INTO genotypes(generation,genotype,imgfile) '+
   'VALUES('+inttostr(thisGeneration)+', '+inttostr(i)+', '''+
    imgName+''');'
   );
   sbStatus.Panels[0].Text:=inttostr(i+1)+'/'+
    inttostr(Population.PopulationSize);
   Application.ProcessMessages;
   If not Working then break; 
 end;
 sbStatus.Panels[0].Text:='Saving SQL queries...';
 Inserts.SaveToFile('sql\sql-g'+inttostr(thisGeneration)+'.sql');
 Inserts.Free;
 Log(inttostr(thisgeneration)+'th generation images & SQL ready.');
 sbStatus.SimpleText:='';
 btStop.Visible:=false;
end;

procedure TfmMain.BitBtn2Click(Sender: TObject);
begin
  Restart;
	if LoadPopulationFromXML('genotypes.xml') then begin
   Log('Genotypes loaded. Generation = '+inttostr(thisGeneration));
  end;
end;

procedure TfmMain.BitBtn3Click(Sender: TObject);
var
 i:integer;
begin
 mmVotes.Clear;
 for i:=0 to Population.PopulationSize-1 do
  mmVotes.Lines.Add(inttostr(i)+'='+inttostr(random(100)));
end;

procedure TfmMain.cbImageSizeChange(Sender: TObject);
begin
 case cbImageSize.ItemIndex of
  0:begin cImageWidth:=200; cImageHeight:=150; end;
  1:begin cImageWidth:=400; cImageHeight:=300; end;
  2:begin cImageWidth:=800; cImageHeight:=600; end;
 end; 
end;

procedure TfmMain.BitBtn4Click(Sender: TObject);
begin
 GenerateImgAndSQL;
end;

procedure TfmMain.cbSubsamplesChange(Sender: TObject);
var
 i,c:integer;
begin
 val(cbSubsamples.Text, i, c);
 if (c=0) and (i>0) then Subsamples:=i;
end;

procedure TfmMain.btStopClick(Sender: TObject);
begin
 Working:=false;
end;

end.

